<?php
	 
	/*
	== Notes About Language files ==
	Each language directory must be named using an ISO-639 language code
	plus an ISO-3166 country code (if any). Codes must be separated by a "-"
	
	i.e.:
		./lang/es		(Means a generic Spanish)
		./lang/es-AR	(Means an Argetina's Spanish)
	
	Note that the format is two lowercase letters (language code) followed
	by a "-" and finally two uppercase letters (country code). Country
	code is completely optional.
	
	When you write a language for Gekko, it is recommended that you change
	the name of the language and country as if it were read by local
	speakers.
	
	i.e.:
		originally:
		"es" => "Spanish"

		becomes:
		"es" => "Espa&ntilde;ol"

		originally:
		"MX" => "Mexico"

		becomes:
		"MX" => "M&eacute;xico"
	*/
	
	/**
	*	ISO 639 Language Codes
	*	Source: http://www.w3.org/WAI/ER/IG/ert/iso639.htm
	*/
	$ISO_639 = Array (
		"aa" => "Afar",
		"ab" => "Abkhazian",
		"af" => "Afrikaans",
		"am" => "Amharic",
		"ar" => "Arabic",
		"as" => "Assamese",
		"ay" => "Aymara",
		"az" => "Azerbaijani",
		"ba" => "Bashkir",
		"be" => "Byelorussian",
		"bg" => "Bulgarian",
		"bh" => "Bihari",
		"bi" => "Bislama",
		"bn" => "Bengali",
		"bo" => "Tibetan",
		"br" => "Breton",
		"ca" => "Catalan",
		"co" => "Corsican",
		"cs" => "Czech",
		"cy" => "Welsh",
		"da" => "Danish",
		"de" => "German",
		"dz" => "Bhutani",
		"el" => "Greek",
		"en" => "English",
		"eo" => "Esperanto",
		"es" => "Espa&ntilde;ol",
		"et" => "Estonian",
		"eu" => "Basque",
		"fa" => "Persian",
		"fi" => "Finnish",
		"fj" => "Fiji",
		"fo" => "Faeroese",
		"fr" => "French",
		"fy" => "Frisian",
		"ga" => "Irish",
		"gd" => "Gaelic",
		"gl" => "Galician",
		"gn" => "Guarani",
		"gu" => "Gujarati",
		"ha" => "Hausa",
		"hi" => "Hindi",
		"hr" => "Croatian",
		"hu" => "Hungarian",
		"hy" => "Armenian",
		"ia" => "Interlingua",
		"ie" => "Interlingue",
		"ik" => "Inupiak",
		"in" => "Indonesian",
		"is" => "Icelandic",
		"it" => "Italian",
		"iw" => "Hebrew",
		"ja" => "Japanese",
		"ji" => "Yiddish",
		"jw" => "Javanese",
		"ka" => "Georgian",
		"kk" => "Kazakh",
		"kl" => "Greenlandic",
		"km" => "Cambodian",
		"kn" => "Kannada",
		"ko" => "Korean",
		"ks" => "Kashmiri",
		"ku" => "Kurdish",
		"ky" => "Kirghiz",
		"la" => "Latin",
		"ln" => "Lingala",
		"lo" => "Laothian",
		"lt" => "Lithuanian",
		"lv" => "Latvian",
		"mg" => "Malagasy",
		"mi" => "Maori",
		"mk" => "Macedonian",
		"ml" => "Malayalam",
		"mn" => "Mongolian",
		"mo" => "Moldavian",
		"mr" => "Marathi",
		"ms" => "Malay",
		"mt" => "Maltese",
		"my" => "Burmese",
		"na" => "Nauru",
		"ne" => "Nepali",
		"nl" => "Dutch",
		"no" => "Norwegian",
		"oc" => "Occitan",
		"om" => "Oromo",
		"or" => "Oriya",
		"pa" => "Punjabi",
		"pl" => "Polish",
		"ps" => "Pashto",
		"pt" => "Portuguese",
		"qu" => "Quechua",
		"rm" => "Rhaeto-Romance",
		"rn" => "Kirundi",
		"ro" => "Romanian",
		"ru" => "Russian",
		"rw" => "Kinyarwanda",
		"sa" => "Sanskrit",
		"sd" => "Sindhi",
		"sg" => "Sangro",
		"sh" => "Serbo-Croatian",
		"si" => "Singhalese",
		"sk" => "Slovak",
		"sl" => "Slovenian",
		"sm" => "Samoan",
		"sn" => "Shona",
		"so" => "Somali",
		"sq" => "Albanian",
		"sr" => "Serbian",
		"ss" => "Siswati",
		"st" => "Sesotho",
		"su" => "Sudanese",
		"sv" => "Swedish",
		"sw" => "Swahili",
		"ta" => "Tamil",
		"te" => "Tegulu",
		"tg" => "Tajik",
		"th" => "Thai",
		"ti" => "Tigrinya",
		"tk" => "Turkmen",
		"tl" => "Tagalog",
		"tn" => "Setswana",
		"to" => "Tonga",
		"tr" => "Turkish",
		"ts" => "Tsonga",
		"tt" => "Tatar",
		"tw" => "Twi",
		"uk" => "Ukrainian",
		"ur" => "Urdu",
		"uz" => "Uzbek",
		"vi" => "Vietnamese",
		"vo" => "Volapuk",
		"wo" => "Wolof",
		"xh" => "Xhosa",
		"yo" => "Yoruba",
		"zh" => "Chinese",
		"zu" => "Zulu"
	);

	/**
	*	ISO 3166 Country Codes
	*	Source: http://www.iso.org/iso/en/prods-services/iso3166ma/index.html
	*/
	$ISO_3166 = Array (
		"AF" => "Afghanistan",
		"AX" => "&Aring;land Islands",
		"AL" => "Albania",
		"DZ" => "Algeria",
		"AS" => "American Samoa",
		"AD" => "Andorra",
		"AO" => "Angola",
		"AI" => "Anguilla",
		"AQ" => "Antarctica",
		"AG" => "Antigua And Barbuda",
		"AR" => "Argentina",
		"AM" => "Armenia",
		"AW" => "Aruba",
		"AU" => "Australia",
		"AT" => "Austria",
		"AZ" => "Azerbaijan",
		"BS" => "Bahamas",
		"BH" => "Bahrain",
		"BD" => "Bangladesh",
		"BB" => "Barbados",
		"BY" => "Belarus",
		"BE" => "Belgium",
		"BZ" => "Belize",
		"BJ" => "Benin",
		"BM" => "Bermuda",
		"BT" => "Bhutan",
		"BO" => "Bolivia",
		"BA" => "Bosnia And Herzegovina",
		"BW" => "Botswana",
		"BV" => "Bouvet Island",
		"BR" => "Brazil",
		"IO" => "British Indian Ocean Territory",
		"BN" => "Brunei Darussalam",
		"BG" => "Bulgaria",
		"BF" => "Burkina Faso",
		"BI" => "Burundi",
		"KH" => "Cambodia",
		"CM" => "Cameroon",
		"CA" => "Canada",
		"CV" => "Cape Verde",
		"KY" => "Cayman Islands",
		"CF" => "Central African Republic",
		"TD" => "Chad",
		"CL" => "Chile",
		"CN" => "China",
		"CX" => "Christmas Island",
		"CC" => "Cocos (keeling) Islands",
		"CO" => "Colombia",
		"KM" => "Comoros",
		"CG" => "Congo",
		"CD" => "Congo, The Democratic Republic Of The",
		"CK" => "Cook Islands",
		"CR" => "Costa Rica",
		"CI" => "Cote D'ivoire",
		"HR" => "Croatia",
		"CU" => "Cuba",
		"CY" => "Cyprus",
		"CZ" => "Czech Republic",
		"DK" => "Denmark",
		"DJ" => "Djibouti",
		"DM" => "Dominica",
		"DO" => "Dominican Republic",
		"EC" => "Ecuador",
		"EG" => "Egypt",
		"SV" => "El Salvador",
		"GQ" => "Equatorial Guinea",
		"ER" => "Eritrea",
		"EE" => "Estonia",
		"ET" => "Ethiopia",
		"FK" => "Falkland Islands (malvinas)",
		"FO" => "Faroe Islands",
		"FJ" => "Fiji",
		"FI" => "Finland",
		"FR" => "France",
		"GF" => "French Guiana",
		"PF" => "French Polynesia",
		"TF" => "French Southern Territories",
		"GA" => "Gabon",
		"GM" => "Gambia",
		"GE" => "Georgia",
		"DE" => "Germany",
		"GH" => "Ghana",
		"GI" => "Gibraltar",
		"GR" => "Greece",
		"GL" => "Greenland",
		"GD" => "Grenada",
		"GP" => "Guadeloupe",
		"GU" => "Guam",
		"GT" => "Guatemala",
		"GN" => "Guinea",
		"GW" => "Guinea-bissau",
		"GY" => "Guyana",
		"HT" => "Haiti",
		"HM" => "Heard Island And Mcdonald Islands",
		"VA" => "Holy See (vatican City State)",
		"HN" => "Honduras",
		"HK" => "Hong Kong",
		"HU" => "Hungary",
		"IS" => "Iceland",
		"IN" => "India",
		"ID" => "Indonesia",
		"IR" => "Iran, Islamic Republic Of",
		"IQ" => "Iraq",
		"IE" => "Ireland",
		"IL" => "Israel",
		"IT" => "Italy",
		"JM" => "Jamaica",
		"JP" => "Japan",
		"JO" => "Jordan",
		"KZ" => "Kazakhstan",
		"KE" => "Kenya",
		"KI" => "Kiribati",
		"KP" => "Korea, Democratic People's Republic Of",
		"KR" => "Korea, Republic Of",
		"KW" => "Kuwait",
		"KG" => "Kyrgyzstan",
		"LA" => "Lao People's Democratic Republic",
		"LV" => "Latvia",
		"LB" => "Lebanon",
		"LS" => "Lesotho",
		"LR" => "Liberia",
		"LY" => "Libyan Arab Jamahiriya",
		"LI" => "Liechtenstein",
		"LT" => "Lithuania",
		"LU" => "Luxembourg",
		"MO" => "Macao",
		"MK" => "Macedonia, The Former Yugoslav Republic Of",
		"MG" => "Madagascar",
		"MW" => "Malawi",
		"MY" => "Malaysia",
		"MV" => "Maldives",
		"ML" => "Mali",
		"MT" => "Malta",
		"MH" => "Marshall Islands",
		"MQ" => "Martinique",
		"MR" => "Mauritania",
		"MU" => "Mauritius",
		"YT" => "Mayotte",
		"MX" => "M&eacute;xico",
		"FM" => "Micronesia, Federated States Of",
		"MD" => "Moldova, Republic Of",
		"MC" => "Monaco",
		"MN" => "Mongolia",
		"MS" => "Montserrat",
		"MA" => "Morocco",
		"MZ" => "Mozambique",
		"MM" => "Myanmar",
		"NA" => "Namibia",
		"NR" => "Nauru",
		"NP" => "Nepal",
		"NL" => "Netherlands",
		"AN" => "Netherlands Antilles",
		"NC" => "New Caledonia",
		"NZ" => "New Zealand",
		"NI" => "Nicaragua",
		"NE" => "Niger",
		"NG" => "Nigeria",
		"NU" => "Niue",
		"NF" => "Norfolk Island",
		"MP" => "Northern Mariana Islands",
		"NO" => "Norway",
		"OM" => "Oman",
		"PK" => "Pakistan",
		"PW" => "Palau",
		"PS" => "Palestinian Territory, Occupied",
		"PA" => "Panama",
		"PG" => "Papua New Guinea",
		"PY" => "Paraguay",
		"PE" => "Peru",
		"PH" => "Philippines",
		"PN" => "Pitcairn",
		"PL" => "Poland",
		"PT" => "Portugal",
		"PR" => "Puerto Rico",
		"QA" => "Qatar",
		"RE" => "Reunion",
		"RO" => "Romania",
		"RU" => "Russian Federation",
		"RW" => "Rwanda",
		"SH" => "Saint Helena",
		"KN" => "Saint Kitts And Nevis",
		"LC" => "Saint Lucia",
		"PM" => "Saint Pierre And Miquelon",
		"VC" => "Saint Vincent And The Grenadines",
		"WS" => "Samoa",
		"SM" => "San Marino",
		"ST" => "Sao Tome And Principe",
		"SA" => "Saudi Arabia",
		"SN" => "Senegal",
		"CS" => "Serbia And Montenegro",
		"SC" => "Seychelles",
		"SL" => "Sierra Leone",
		"SG" => "Singapore",
		"SK" => "Slovakia",
		"SI" => "Slovenia",
		"SB" => "Solomon Islands",
		"SO" => "Somalia",
		"ZA" => "South Africa",
		"GS" => "South Georgia And The South Sandwich Islands",
		"ES" => "Espa&ntilde;a",
		"LK" => "Sri Lanka",
		"SD" => "Sudan",
		"SR" => "Suriname",
		"SJ" => "Svalbard And Jan Mayen",
		"SZ" => "Swaziland",
		"SE" => "Sweden",
		"CH" => "Switzerland",
		"SY" => "Syrian Arab Republic",
		"TW" => "Taiwan, Province Of China",
		"TJ" => "Tajikistan",
		"TZ" => "Tanzania, United Republic Of",
		"TH" => "Thailand",
		"TL" => "Timor-leste",
		"TG" => "Togo",
		"TK" => "Tokelau",
		"TO" => "Tonga",
		"TT" => "Trinidad And Tobago",
		"TN" => "Tunisia",
		"TR" => "Turkey",
		"TM" => "Turkmenistan",
		"TC" => "Turks And Caicos Islands",
		"TV" => "Tuvalu",
		"UG" => "Uganda",
		"UA" => "Ukraine",
		"AE" => "United Arab Emirates",
		"GB" => "United Kingdom",
		"US" => "United States",
		"UM" => "United States Minor Outlying Islands",
		"UY" => "Uruguay",
		"UZ" => "Uzbekistan",
		"VU" => "Vanuatu",
		"VE" => "Venezuela",
		"VN" => "Viet Nam",
		"VG" => "Virgin Islands, British",
		"VI" => "Virgin Islands, U.s.",
		"WF" => "Wallis And Futuna",
		"EH" => "Western Sahara",
		"YE" => "Yemen",
		"ZM" => "Zambia",
		"ZW" => "Zimbabwe"
	);

	$lang_db = Array();
	
	// Fetching available language files
	$files = listdir("lang");

	foreach ($files as $file) {
		$abs_path = GEKKO_SOURCE_DIR."lang/{$file}";
		if (is_dir($abs_path)) {
		if ((strpos($file, "-")) !== false) {
				// language-COUNTRY:
				// i.e.: en-US, es-MX, es-AR
				list($code, $country) = explode("-", $file);
				$lang_db[$file] = (isset($ISO_639[$code]) ? $ISO_639[$code] : $code)." (".(isset($ISO_3166[$country]) ? $ISO_3166[$country] : $contry).")";
			} else {
				// generic language
				$lang_db[$file] = isset($ISO_639[$file]) ? $ISO_639[$file] : $file;
			}
		}
	}
?>